# -*- coding: utf-8 -*-
"""
@author: Svenja C. Hövelmann, Kiel University and Deutsches Elektronen-Synchrotron DESY

last edit: 28 February 2024

Read nexus files SAXS DELTA BL2 data
"""
import h5py
import matplotlib.pyplot as plt


#%% define class
class read_nexus_file(object):
    def __init__(self, filename):
        self.filename = filename
    
    def print_information(self):
        with h5py.File(self.filename, 'r') as f:
            print("Nexus file informations")
            
            print("\n Principal investigator")
            for key in f["entry/principal investigator/"].keys():
                if isinstance(f["entry/principal investigator/"][key][()], bytes):
                    print(key+": "+f["entry/principal investigator/"][key][()].decode())
                else:
                    print(key+": "+str(f["entry/principal investigator/"][key][()]))
                for item in f["entry/principal investigator/"][key].attrs.keys():
                    print(item + ":", f["entry/principal investigator/"][key].attrs[item])
            
            print("\n Instrument information")
            for key in f["entry/instrument/"].keys():
                if isinstance(f["entry/instrument/"][key][()], bytes):
                    print(key+": "+f["entry/instrument/"][key][()].decode())
                else:
                    print(key+": "+str(f["entry/instrument/"][key][()]))
                for item in f["entry/instrument/"][key].attrs.keys():
                    print(item + ":", f["entry/instrument/"][key].attrs[item])
            
            print("\n Detector information")
            for key in f["entry/detector/"].keys():
                if isinstance(f["entry/detector/"][key][()], bytes):
                    print(key+": "+f["entry/detector/"][key][()].decode())
                else:
                    print(key+": "+str(f["entry/detector/"][key][()]))
                for item in f["entry/detector/"][key].attrs.keys():
                    print(item + ":", f["entry/detector/"][key].attrs[item])
              
            print("\n Extraction settings")
            for key in f["entry/extraction settings/"].keys():
                if isinstance(f["entry/extraction settings/"][key][()], bytes):
                    print(key+": "+f["entry/extraction settings/"][key][()].decode())
                else:
                    print(key+": "+str(f["entry/extraction settings/"][key][()]))
                for item in f["entry/extraction settings/"][key].attrs.keys():
                    print(item + ":", f["entry/extraction settings/"][key].attrs[item])
            
            print("\n Sample information")
            for key in f["entry/sample/"].keys():
                if isinstance(f["entry/sample/"][key][()], bytes):
                    print(key+": "+f["entry/sample/"][key][()].decode())
                else:
                    print(key+": "+str(f["entry/sample/"][key][()]))
                for item in f["entry/sample/"][key].attrs.keys():
                    print(item + ":", f["entry/sample/"][key].attrs[item])
        f.close()

    def plot(self):
        with h5py.File(self.filename, "r") as f:
            q       = f["/entry/data/q"][:]
            inten   = f["/entry/data/intensity"][:]
            plt.figure()
            plt.plot(q,inten, "k")
            plt.xlabel("q / nm$^{-1}$")
            plt.ylabel("intensity")
            plt.tight_layout()
            plt.show()
        f.close()

    def get_data(self):
        with h5py.File(self.filename, "r") as f:
            q       = f["/entry/data/q"][:]
            inten   = f["/entry/data/intensity"][:]
        f.close()
        return q, inten
    
    def get_sampleinfo(self):
        with h5py.File(self.filename, "r") as f:
            text = "Sample information \n"
            for key in f["entry/sample/"].keys():
                if isinstance(f["entry/sample/"][key][()], bytes):
                    text += (key+": "+f["entry/sample/"][key][()].decode())
                else:
                    text += (key+": "+str(f["entry/sample/"][key][()]))
                for item in f["entry/sample/"][key].attrs.keys():
                    text += (" "+f["entry/sample/"][key].attrs[item])
                text += "\n"
        f.close()
        return text

file = "2022-03/bg_corrected_v3/dppc2_5mm_mlv_wasser_37p0_002_001.nx5"
read_nexus_file(file).print_information()
