# -*- coding: utf-8 -*-
"""
@author: Svenja C. Hövelmann, Kiel University and Deutsches Elektronen-Synchrotron DESY

last edit: 15 May 2024

Write nexus files SAXS DELTA BL2 data
"""
import h5py

class write_nexus_file(object):
    def __init__(self, filename, q, intensity, data_file, bg_file, det_info, batch_info, instrument_info, investigator_info, cur, cur_file, bg_cur, bg_cur_file, bg_factor, sample_info):
        self.filename = filename
        self.q = q
        self.inten = intensity
        self.data_file = data_file
        self.bg_file = bg_file
        self.det_info = det_info
        self.batch_info = batch_info        
        self.instrument_info = instrument_info
        self.investigator_info = investigator_info
        self.cur = cur
        self.cur_file = cur_file
        self.bg_cur = bg_cur
        self.bg_cur_file = bg_cur_file
        self.bg_factor = bg_factor
        self.sample_info = sample_info        
    
    def __call__(self):#, filename, q, intensity, data_file, bg_file, det_info, batch_info, cur, bg_cur, bg_factor):
        with h5py.File(self.filename, 'w') as f:
              f.attrs['default'] = 'entry'
          #first level
              nxentry = f.create_group('entry')
              nxentry.attrs["NX_class"] = 'NXentry'
              nxentry.attrs['default'] = 'data'
        
          #instrument information
              nxinstrument = nxentry.create_group("instrument")
              nxinstrument.attrs["NX_class"] = 'NXinstrument'
              
              facility = nxinstrument.create_dataset("facility", data = self.instrument_info["facility"])
              beamline = nxinstrument.create_dataset("beamline", data = self.instrument_info["beamline"])
              beamtime = nxinstrument.create_dataset("beamtime", data = self.instrument_info["beamtime"])
        
         # invesigator information
              investigator = nxentry.create_group("principal investigator")
              first_name = investigator.create_dataset("first name", data = self.investigator_info["firstname"])
              last_name = investigator.create_dataset("last name", data = self.investigator_info["lastname"])
              affiliation = investigator.create_dataset("affiliation", data = self.investigator_info["affiliation"])
              ORCID = investigator.create_dataset("ORCID", data = self.investigator_info["ORCID"])
             
          # store data in nxdetector group
              nxdetector = nxentry.create_group("detector")
              nxdetector.attrs["NX_class"] = 'NXdetector'
              # data
              q           = nxdetector.create_dataset("q", data = self.q)
              q.attrs["units"] = "inverse nanometer"
              inten       = nxdetector.create_dataset("intensity", data = self.inten)
              inten.attrs["units"] = "counts"
              inten.attrs["current"] = self.cur
              inten.attrs["normalised"] = "current"
              inten.attrs["background corrected"] = "yes"
              
              bg_file = nxdetector.create_dataset("background file", data = self.bg_file)
              bg_file.attrs["bg normalised by"] = "current"
              bg_file.attrs["bg current file"] = self.bg_cur_file
              bg_file.attrs["bg current"] = self.cur
              bg_file.attrs["bg factor"] = self.bg_factor    
              data_file   = nxdetector.create_dataset("data file", data = self.data_file)
              cur_file    = nxdetector.create_dataset("current file", data = self.cur_file)
              
              # detector information
              energy      = nxdetector.create_dataset("energy", data = self.det_info["energy"])
              energy.attrs["units"] = "eV"
              detector    = nxdetector.create_dataset("detector", data = self.det_info["detector"])
              detector_file = nxdetector.create_dataset("detector file", data = self.data_file+".mar345")
              count_time  = nxdetector.create_dataset("count time", data = self.det_info["count_time"])
              count_time.attrs["units"] = "seconds" 
             
              # extraction data
              nxextract   = nxentry.create_group("extraction settings")   # extraction information
              distance    = nxextract.create_dataset("detector distance", data = self.batch_info["det_dis"])
              distance.attrs["units"] = "m"
              wavelength  = nxextract.create_dataset("wavelength", data = self.batch_info["wavelength"])
              wavelength.attrs["units"] = "Angstrom"
              mask_file   = nxextract.create_dataset("mask file", data = self.batch_info["mask"])
              px_x_size   = nxextract.create_dataset("pixel size x", data = self.batch_info["px_x_size"])
              px_y_size   = nxextract.create_dataset("pixel size y", data = self.batch_info["px_x_size"])
              beam_x_cen  = nxextract.create_dataset("beam center x", data = self.batch_info["beam_x_cen"])
              beam_y_cen  = nxextract.create_dataset("beam center y", data = self.batch_info["beam_y_cen"])
              tilt_rot    = nxextract.create_dataset("tilt rotation", data = self.batch_info["tilt_rot"])
              tilt_rot.attrs["units"] = "degrees"
              angle_tilt  = nxextract.create_dataset("angle of tilt", data = self.batch_info["angle_tilt"])
              angle_tilt.attrs["units"] = "degrees"
              bins        = nxextract.create_dataset("bins", data = self.batch_info["bins"])
              
              nxsample = nxentry.create_group("sample")
              nxsample.attrs["NX_class"] = "NXsample"
              molecules     = nxsample.create_dataset("molecules", data = self.sample_info["molecules"])
              IGSN           = nxsample.create_dataset("IGSN", data = self.sample_info["IGSN"])
              concentration = nxsample.create_dataset("concentration", data = self.sample_info["concentration"])
              concentration.attrs["units"] = "mMol/l"
              ratio_AZ      = nxsample.create_dataset("ratio AZ", data = self.sample_info["ratio_AZ"])
              ratio_AZ.attrs["units"] = "%"
              switching     = nxsample.create_dataset("isomere state", data = self.sample_info["switching_state"])
              temperature   = nxsample.create_dataset("temperature", data = self.sample_info["temperature"])
              temperature.attrs["units"] = "Celsius"
              comment       = nxsample.create_dataset("comment", data = self.sample_info["comment"])
             

            # define default data plot
              nxdata = nxentry.create_group('data')   
              nxdata.attrs["NX_class"] = 'NXdata'
              nxdata.attrs['signal'] = 'counts'
              nxdata.attrs['axes'] = 'q (nm-1)'
        
              source_addr = "entry/instrument/detector/q"
              target_addr = "q"
              q.attrs["target"] = source_addr
              nxdata[target_addr] = f[source_addr]
              
              source_addr = "entry/instrument/detector/intensity"
              target_addr = "intensity"
              inten.attrs["target"] = source_addr
              nxdata[target_addr] = f[source_addr]
        f.close()
